/**
  Dieses Ctrl Skript ist verantwortlich fuer die Kommunikation zwischen WinCC OA 
  und dem Barco OpSpace System
  
  @author: Michael Schmit (BECC GmbH)
  @date:   2018
*/

#uses "stdTraffic_intern"
#uses "barco"
#uses "dist"

global int BARCO_SIMPLE_API_TCP_SOCKET = -99;
global bool gbReduCBInitialised = false;
global int giReconnectThreadId = -1;

global dyn_string gdsWorkstations;
global mapping    gmWorkstationMapping;

global mapping    gmCamActiveStateCache;

/** 
  Main Funktion
  @param ... Arbeitsplatzrechner hostname 
*/
void main(...)
{
  va_list vaParameter;
  
  int iParameterLen = va_start(vaParameter);
  
  string sConfig = getPath(DATA_REL_PATH, "barco/barco.cfg");
  
  if ( !isfile(sConfig) )
  {
    stdTraffic_intern_logWarning(__FILE__, "Konfig-Datei existiert nicht: " + "barco/barco.cfg");
    return;
  }

  for ( int i = 1; i <= iParameterLen; i++ )//Parameter einlesen
  {
    string sWorkstation = strtolower(va_arg(vaParameter));
    
    dynAppend(gdsWorkstations, sWorkstation);
    
    gmWorkstationMapping[stdTraffic_intern_paCfgReadValue(sConfig, "workspace_mapping", sWorkstation)] = sWorkstation;    
  }
  va_end(vaParameter);
  
  if ( dynlen(gdsWorkstations) == 0 )//kein arbeitsplatz angegeben
  {
    stdTraffic_intern_logWarning(__FILE__, "Kein Arbeitsplatz uebergeben beende mich");
    return;
  }
  
  barco_initVars();//variablen initialisieren
  
  //auf redundanzumschaltungen connecten
  dpConnect("barco_manager_reduCB", "_ReduManager.Status.Active",
                                    "_ReduManager_2.Status.Active");
  
  while ( !gbReduCBInitialised )//warte bis redu calback fertig ausgefuehrt wurde
    delay(1);
  
  //workstation datenpunkte anlegen wenn noch nicht vorhanden
  for ( int i = 1; i <= dynlen(gdsWorkstations); i++ )
  {
    string sDp = "barco_workstation_" + gdsWorkstations[i];
    
    if ( !dpExists(sDp) )
      dpCreate(sDp, "barco_workstation");
    
    dpSetWait(sDp+".workspace.name", "");//zuweisung am anfang loeschen
    
    //lausche auf request
    dpConnect("barco_manager_commandCB", false,  sDp+".request.command",
                                                 sDp+".request.user",
                                                 sDp+".request.manNum",
                                                 sDp+".request.manType",
                                                 sDp+".request.REST_CMD");
    
    barco_executeCommand(gdsWorkstations[i], "OpsAPI "+BARCO_OWS_IP+" "+BARCO_OWS_USER+" "+BARCO_OWS_PASSWORD);
    
    dpSet("barco_workstation_"+gdsWorkstations[i]+".workspace.alarms.source", makeDynString(),
          "barco_workstation_"+gdsWorkstations[i]+".workspace.alarms.prio", makeDynInt(),
          "barco_workstation_"+gdsWorkstations[i]+".workspace.alarms.direction", makeDynInt(),
          "barco_workstation_"+gdsWorkstations[i]+".workspace.alarms.dp", makeDynString(),
          "barco_workstation_"+gdsWorkstations[i]+".workspace.alarms.text", makeDynString());
  } 
  
  //periodisch sourcen synchronisieren
//   startThread("barco_manager_syncSources");
//   delay(2);
  
  //periodisch workspaces synchronisieren
  startThread("barco_manager_syncWorkspaces");
  delay(2);
  
  //periodischer update der workstation ansicht
  startThread("barco_manager_syncVisualObjects");
  delay(2);
  
  //periodischer update der layouts
  startThread("barco_manager_syncLayouts");
  delay(2);
  
  //Kamera aktiv status cache
  dpQueryConnectSingle("barco_manager_camActiveStateCacheCB", true, "cam", "SELECT '_original.._value' FROM '*_tv_*.meldung.aktiv'");
  
  //angezeigte visual objects
  dpQueryConnectAll("barco_manager_shownCamsCB", true, "cam", "SELECT '_original.._value' FROM 'barco_workstation_*.workspace.visualObjects.sourceName'");
  
  //angezeigte ivz visual objects
  dpQueryConnectAll("barco_manager_shownIvzCamsCB", true, "cam", "SELECT '_original.._value' FROM 'a09vbz_tv_ivz_monitor_*.istwert.kamera'");
}

/** 
  Periodischer ableich der visual objects
*/
void barco_manager_syncVisualObjects()
{
  while(true)//alle 10 sec sourcen syncronisieren
  {
    for ( int i = 1; i <= dynlen(gdsWorkstations); i++ )
      barco_updateWorkstationOverview(gdsWorkstations[i]);
    
    delay(10);
  }
}

/** 
  Periodischer ableich der layouts
*/
void barco_manager_syncLayouts()
{
  mapping mLayouts;
  
  while(true)//alle 10 min sourcen syncronisieren
  {
    for ( int i = 1; i <= dynlen(gdsWorkstations); i++ )
    {
      mLayouts[gdsWorkstations[i]] = makeDynString();
    }
    
    dyn_string dsLayouts = barco_getLayouts(gdsWorkstations[1]);//nur auf einem ausfuehren da alle sourcen fuer jeden gleich sind    
    
    for ( int i = 1; i <= dynlen(dsLayouts); i++ )
    {
      string sWorkspace = barco_getLayoutWorkspace(gdsWorkstations[1], dsLayouts[i]);
      dynAppend(mLayouts[barco_getWorkstationNameOfWorkspace(sWorkspace)], dsLayouts[i]);
    }
    
    dyn_string dsKeys = mappingKeys(mLayouts);
    
    for ( int i = 1; i <= dynlen(dsKeys); i++ )
    {
      string sDp = "barco_workstation_"+dsKeys[i];
      
      barco_dpSetOldNewComp(sDp+".workspace.layouts", mLayouts[dsKeys[i]]);
    }

    delay(10*60);
  }
}

/** 
  Periodischer ableich der workspaces
*/
void barco_manager_syncWorkspaces()
{
  while(true)//alle 10 min sourcen syncronisieren
  {
    dyn_string dsWorkspaces = barco_getWorkspaces(gdsWorkstations[1]);//nur auf einem ausfuehren da alle sourcen fuer jeden gleich sind
        
    for ( int i = 1; i <= dynlen(dsWorkspaces); i++ )
    {
      string sHost = gmWorkstationMapping[dsWorkspaces[i]];
      string sDp = "barco_workstation_"+sHost;
      
      dyn_int diSize = barco_getWorkspaceSize(gdsWorkstations[1], dsWorkspaces[i]);
      
     
      barco_dpSetOldNewComp(sDp+".workspace.name", dsWorkspaces[i]);

      barco_dpSetOldNewComp(sDp+".workspace.size.height", diSize[2]);
      barco_dpSetOldNewComp(sDp+".workspace.size.width", diSize[1]);
    }
    
    delay(10*60);
  }
}

/** 
  Periodischer ableich der sourcen
*/
void barco_manager_syncSources()
{
  while(true)//alle 10 min sourcen syncronisieren
  {
    dyn_mapping dmSources = barco_getSources(gdsWorkstations[1]);//nur auf einem ausfuehren da alle sourcen fuer jeden gleich sind
    
    for ( int i = 1; i <= dynlen(dmSources); i++ )
    {
      string sId = dmSources[i]["_id"];
      string sDp = "barco_source_"+sId;
      
      if ( !dpExists(sDp) )//source dp anlegen wenn er nicht existiert
        dpCreate(sDp, "barco_source");
      
      barco_dpSetOldNewComp(sDp+".streamUrl", dmSources[i]["streamUrl"]);
      barco_dpSetOldNewComp(sDp+".exclusiveLockOwner", dmSources[i]["exclusiveLockOwner"]);
      barco_dpSetOldNewComp(sDp+".location", dmSources[i]["location"]);
      barco_dpSetOldNewComp(sDp+".componentType", dmSources[i]["componentType"]);
      barco_dpSetOldNewComp(sDp+".mediaGroup", dmSources[i]["mediaGroup"]);
      barco_dpSetOldNewComp(sDp+".isFavorite", dmSources[i]["isFavorite"]);
      barco_dpSetOldNewComp(sDp+".gwUrl", dmSources[i]["gwUrl"]);
      barco_dpSetOldNewComp(sDp+".canBeExclusive", dmSources[i]["canBeExclusive"]);
      barco_dpSetOldNewComp(sDp+"._id", dmSources[i]["_id"]);
      barco_dpSetOldNewComp(sDp+".sourceType", dmSources[i]["sourceType"]);
      barco_dpSetOldNewComp(sDp+".description", dmSources[i]["description"]);
      barco_dpSetOldNewComp(sDp+".componentObjectId", dmSources[i]["componentObjectId"]);
      barco_dpSetOldNewComp(sDp+".name", dmSources[i]["name"]);
      barco_dpSetOldNewComp(sDp+".isPublic", dmSources[i]["isPublic"]);
      barco_dpSetOldNewComp(sDp+".mediaUrl", dmSources[i]["mediaUrl"]);
      barco_dpSetOldNewComp(sDp+".url", dmSources[i]["url"]);
      
      if ( mappingHasKey(dmSources[i], "options") )
      {
        barco_dpSetOldNewComp(sDp+".options.mode", dmSources[i]["options"]["mode"]);
        barco_dpSetOldNewComp(sDp+".options.volume", dmSources[i]["options"]["volume"]);
        barco_dpSetOldNewComp(sDp+".options.showLabel", dmSources[i]["options"]["showLabel"]);
        barco_dpSetOldNewComp(sDp+".options.mute", dmSources[i]["options"]["mute"]);
        barco_dpSetOldNewComp(sDp+".options.zoom", dmSources[i]["options"]["zoom"]);
        barco_dpSetOldNewComp(sDp+".options.unlock", dmSources[i]["options"]["unlock"]);
        
        if ( mappingHasKey(dmSources[i]["options"], "pointOfInterest") )
        {
          barco_dpSetOldNewComp(sDp+".options.pointOfInterest.left", dmSources[i]["options"]["pointOfInterest"]["left"]);
          barco_dpSetOldNewComp(sDp+".options.pointOfInterest.top", dmSources[i]["options"]["pointOfInterest"]["top"]);
        }
        
      }

      if ( mappingHasKey(dmSources[i], "decoderConfig") && getTypeName(dmSources[i]["decoderConfig"]) == "mapping" )
      {
        if ( mappingHasKey(dmSources[i]["decoderConfig"], "latency") )
          barco_dpSetOldNewComp(sDp+".decoderConfig.latency", dmSources[i]["decoderConfig"]["latency"]);
        if ( mappingHasKey(dmSources[i]["decoderConfig"], "use-hardware-postprocessing") )
          barco_dpSetOldNewComp(sDp+".decoderConfig.use-hardware-postprocessing", dmSources[i]["decoderConfig"]["use-hardware-postprocessing"]);
        if ( mappingHasKey(dmSources[i]["decoderConfig"], "use-hardware-decoding") )
          barco_dpSetOldNewComp(sDp+".decoderConfig.use-hardware-decoding", dmSources[i]["decoderConfig"]["use-hardware-decoding"]);
        if ( mappingHasKey(dmSources[i]["decoderConfig"], "sync") )
          barco_dpSetOldNewComp(sDp+".decoderConfig.sync", dmSources[i]["decoderConfig"]["sync"]);
      }
      
    }    
    
    delay(10*60);
  }
}

/** 
  callback welche die befehle abhandelt
  @param sDp1 
  @param sCmd ... befehl
  @param sDp2 
  @param sUser ... user
  @param sDp3 
  @param uManNum ... man num
  @param sDp4 
  @param uManType ... man type
*/
void barco_manager_commandCB(string sDp1, string sCmd,
                             string sDp2, string sUser,
                             string sDp3, uint uManNum,
                             string sDp4, uint uManType,
                             string sDp5, bool bREST)
{  
  string sResult;
  string sRetCmd = sCmd;
  string sWorkstationDp = dpSubStr(sDp1, DPSUB_DP);
  
  if ( bREST )//REST command
  {
    mapping mResult;
    mapping mHeaders = makeMapping("Content-Type", "application/json", "Authorization", "Basic "+base64Encode(BARCO_OWS_USER+":"+BARCO_OWS_PASSWORD)+"");
    
    sCmd = "https://"+BARCO_OWS_IP+"/opspace/api/v1/"+sCmd;
    
    netGet(sCmd, mResult, makeMapping("ignoreSslErrors","", "headers", mHeaders));
    
    if ( mappingHasKey(mResult, "content") && mResult["content"] != "" && !mappingHasKey(mResult, "errorString") )//erfolgreich
    {
      sResult = mResult["content"];
    }
    else//felher aufgetreten
    {
      stdTraffic_intern_logWarning(__FILE__, "REST Command Fehler ("+sCmd+"):"+mResult["errorString"]); 
      return;
    }
  }
  else//kein REST command
  {
    if ( BARCO_SIMPLE_API_TCP_SOCKET == -99 )//nicht verbunden
      return;
  
    sCmd += BARCO_END_CHAR;

    tcpWrite(BARCO_SIMPLE_API_TCP_SOCKET, sCmd);//befehl senden
  
    if ( dynlen(getLastError()) > 0 )//verbindung unterbrochen
    {
      throwError(getLastError());
      
      
    
      if ( giReconnectThreadId == -1 )
        startThread("barco_manager_reconnectCB");
    
      return;
    }
  
    time tStart = getCurrentTime();

    while ( strpos(sResult, BARCO_END_PROMT) < 0 &&//solange lesen bis endprompt angekommen ist
            (getCurrentTime() - tStart) < 60)//und maximal 60 sec
    {
      string sTempResult;

      tcpRead(BARCO_SIMPLE_API_TCP_SOCKET, sTempResult, 5);//antwort abwarten max 5 sec
    
      sResult += sTempResult;   
    }
  
    if ( (getCurrentTime() - tStart) >= 60 )
      stdTraffic_intern_logWarning(__FILE__, "Timeout erreicht ("+sCmd+")");  
  }
  
  
  dpSet(sWorkstationDp+".response.command", sRetCmd,
        sWorkstationDp+".response.result", sResult,
        sWorkstationDp+".response.user", sUser,
        sWorkstationDp+".response.manNum", uManNum,
        sWorkstationDp+".response.manType", uManType);
  
  dpSet(sWorkstationDp+".response.command", "",
        sWorkstationDp+".response.result", "",
        sWorkstationDp+".response.user", "",
        sWorkstationDp+".response.manNum", 0,
        sWorkstationDp+".response.manType", 0);
}

/** 
  Redu callback funktion
  oeffnet am aktiven rechner die tcp verbindung und schliesst die verbindung auf dem passiven rechner 
*/
void barco_manager_reduCB(string sDp,   bool bActive,
                          string sDp_2, bool bActive_2)
{
  int iMyHostNum = myReduHostNum();
  int iActiveHostNum;
  
  reduActive(iActiveHostNum);
  
  if ( iMyHostNum == 2 && iActiveHostNum == 2 )//host 2 aktiv und ich bin host 2
  {
    if ( BARCO_SIMPLE_API_TCP_SOCKET == -99 )//noch nicht verbunden
    {
      BARCO_SIMPLE_API_TCP_SOCKET = tcpOpen(BARCO_OWS_IP, BARCO_OWS_PORT);
      
      string sTemp;
      tcpRead(BARCO_SIMPLE_API_TCP_SOCKET, sTemp, 5);//notwendig willkommensnachricht
      
      if ( dynlen(getLastError()) > 0 )
      {
        throwError(getLastError());
        stdTraffic_intern_logWarning(__FILE__, "Verbindung konnte nicht hergestellt werden auf Host " + iMyHostNum+". beende micht");
        exit();
      }
      else
        stdTraffic_intern_logInfo(__FILE__, "Verbindung hergestellt auf Host " + iMyHostNum);
    }
  }
  else if ( iMyHostNum == 1 && (iActiveHostNum == 1 || iActiveHostNum == 0) )//host 1 aktiv und ich bin host 1
  {
    if ( BARCO_SIMPLE_API_TCP_SOCKET == -99 )//noch nicht verbunden
    {
      BARCO_SIMPLE_API_TCP_SOCKET = tcpOpen(BARCO_OWS_IP, BARCO_OWS_PORT);
      
      string sTemp;
      tcpRead(BARCO_SIMPLE_API_TCP_SOCKET, sTemp, 5);//notwendig willkommensnachricht
      
      if ( dynlen(getLastError()) > 0 )
      {
        throwError(getLastError());
        stdTraffic_intern_logWarning(__FILE__, "Verbindung konnte nicht hergestellt werden auf Host " + iMyHostNum+". beende micht");
        exit();
      }
      else
        stdTraffic_intern_logInfo(__FILE__, "Verbindung hergestellt auf Host " + iMyHostNum);
    }
  }
  else
  {
    tcpClose(BARCO_SIMPLE_API_TCP_SOCKET);
    BARCO_SIMPLE_API_TCP_SOCKET = -99;
    
    stdTraffic_intern_logInfo(__FILE__, "Verbindung geschlossen auf Host " + iMyHostNum);
  }
  
  if ( isAnswer() )
    gbReduCBInitialised = true;
}

/** 
  Ueberprueft ob die Verbindung stehet. reconnect wenn verbindung wieder hergestellt wird
*/
void barco_manager_reconnectCB()
{
  giReconnectThreadId = getThreadId();
  
  while ( true )
  {
    tcpClose(BARCO_SIMPLE_API_TCP_SOCKET);
    
    BARCO_SIMPLE_API_TCP_SOCKET = tcpOpen(BARCO_OWS_IP, BARCO_OWS_PORT);

    string sTemp;
    int j = tcpRead(BARCO_SIMPLE_API_TCP_SOCKET, sTemp, 5);//notwendig willkommensnachricht

    dyn_errClass dec = getLastError();
    
    if ( dynlen(dec) > 0 )//verbindung konnte nicht aufgebaut werden
    {
      throwError(dec);
      stdTraffic_intern_logWarning(__FILE__, "Verbindung zu "+BARCO_OWS_IP+":"+BARCO_OWS_PORT+" unterbrochen");
    }
    else//verbindugn wieder aufgebaut
    {
      for ( int i = 1; i <= dynlen(gdsWorkstations); i++ )
      {
        barco_executeCommand(gdsWorkstations[i], "OpsAPI "+BARCO_OWS_IP+" "+BARCO_OWS_USER+" "+BARCO_OWS_PASSWORD);
      }
      
      stdTraffic_intern_logInfo(__FILE__, "Verbindung wieder hergestellt zu: "+BARCO_OWS_IP+":"+BARCO_OWS_PORT);
      giReconnectThreadId = -1;
      return;
    }  
  
    delay(3);
  }
}

/** 
  @param aUserData 
  @param dda 
*/
void barco_manager_shownIvzCamsCB(anytype aUserData, dyn_dyn_anytype dda)
{
  dyn_string dsShownCamDps;
  
  for ( int i = 2; i <= dynlen(dda); i++ )
  {
    string sSollwertDp = dpSubStr(dda[i][1], DPSUB_DP_EL);
    
    strreplace(sSollwertDp, ".istwert.", ".sollwert.");
    
    string sTemp;
    
    dpGet(sSollwertDp, sTemp);
    
    if ( sTemp == dda[i][2]  && sTemp != "" )
      dynAppend(dsShownCamDps, sTemp);
  }
 
  for ( int i = 1; i <= mappinglen(gmCamActiveStateCache); i++ )
  {
    string sDp = mappingGetKey(gmCamActiveStateCache, i);
    bool bValue = mappingGetValue(gmCamActiveStateCache, i);
    
    if ( strpos(sDp, "_ivz_") != -1 )//ivz kameras werden hier behandelt
    {
      if ( dynContains(dsShownCamDps, sDp) )//sichtbare kamera
      {
        if ( !bValue )
          dpSet(sDp+".meldung.aktiv", true);
      }
      else if ( bValue )
        dpSet(sDp+".meldung.aktiv", false);
    }
      
  }
}

/** 
  @param aUserData 
  @param dda 
*/
void barco_manager_shownCamsCB(anytype aUserData, dyn_dyn_anytype dda)
{
  dyn_string dsShownCamDps;
  
  for ( int i = 2; i <= dynlen(dda); i++ )
  {
    dyn_string dsShownSources = dda[i][2];
    
    for ( int j = 1; j <= dynlen(dsShownSources); j++ )
    {
      string sTemp = barco_getCamDpOfSourceName(dsShownSources[j]);

      if ( sTemp != "" )
        dynAppend(dsShownCamDps, sTemp);
    }
  }

  for ( int i = 1; i <= mappinglen(gmCamActiveStateCache); i++ )
  {
    string sDp = mappingGetKey(gmCamActiveStateCache, i);
    bool bValue = mappingGetValue(gmCamActiveStateCache, i);
    
    if ( strpos(sDp, "_ivz_") != -1 )//ivz kameras werden hier nicht behandelt
      continue;

    if ( dynContains(dsShownCamDps, sDp) )//sichtbare kamera
    {
      if ( !bValue )
        dpSet(sDp+".meldung.aktiv", true);
    }
    else if ( bValue )
      dpSet(sDp+".meldung.aktiv", false);
      
  }
}

/** 
  @param aUserData 
  @param dda 
*/
void barco_manager_camActiveStateCacheCB(anytype aUserData, dyn_dyn_anytype dda)
{
  for ( int i = 2; i <= dynlen(dda); i++ )
  {
    gmCamActiveStateCache[dpSubStr(dda[i][1], DPSUB_DP)] = dda[i][2];
  }
}
