/**
  Dieses Ctrl Skript ist verantwortlich fuer die Alarmrahmen und Selektionsrahmen Aufschaltung auf dem
  Barco OpSpace System
  
  @author: Michael Schmit (BECC GmbH)
  @date:   2018
*/

#uses "barco"

global mapping BARCO_SELECTED_VO;
global mapping BARCO_SELECTED_LAYOUT;

global mapping gmShownAlerts;

void main()
{
  //variablen initialisieren
  barco_initVars();
  
  dyn_string dsWorkstationDps = dpNames("*", "barco_workstation");
  

  for ( int i = 1; i <= dynlen(dsWorkstationDps); i++ ) 
  { 
    dpConnect("barco_video_matrix_frameControllerCB", dsWorkstationDps[i]+".workspace.visualObjects.selected",
                                                      dsWorkstationDps[i]+".workspace.selectedLayout");
  }
  
  
  //Videomatrix starten
  dyn_string dsEreignisDps = mappingKeys(BARCO_VIDEO_MATRIX);
  
  for ( int i = 1; i <= dynlen(dsEreignisDps); i++ )
  {
    string sEreignisDp = dsEreignisDps[i];
    
    dpConnectUserData("barco_video_matrix_videoMatrixCB", sEreignisDp, false, sEreignisDp+":_alert_hdl.._act_state_color",
                                                                              sEreignisDp+":_alert_hdl.._act_state",
                                                                              sEreignisDp+":_alert_hdl.._act_prior");
  }
}

/** 
  work funktion video matrix
  @param aIdent 
  @param sDp1 
  @param sColor 
  @param sDp2 
  @param iActState 
*/
void barco_video_matrix_videoMatrixCB(anytype aIdent, string sDp1, string sColor,
                                                      string sDp2, int iActState,
                                                      string sDp3, int iPrio)
{
  string sAlarmDp = dpSubStr(aIdent, DPSUB_DP_EL);
  string sTunnel = strsplit(sAlarmDp, "_")[1]; 

  dyn_dyn_anytype dda;
  string sAlarmSource = barco_getSourceNameOfCamDp(BARCO_VIDEO_MATRIX[sAlarmDp]["CAM_DP"]);


  string sFrameSource = BARCO_VO_ALARM;
  
  dpQuery("SELECT ALERT '_alert_hdl.._prior', '_alert_hdl.._last' FROM '"+sAlarmDp+"' WHERE '_alert_hdl.._last' == 1",dda);

  if ( dynlen(dda) > 1 )//alarm noch vorhanden
    iPrio = dda[2][3];
  
  if ( iPrio >= BARCO_ALARM_PRIO_MIN && iPrio <= BARCO_ALARM_PRIO_MAX )
    sFrameSource = BARCO_VO_ALARM;
  else if ( iPrio >= BARCO_WARNING_PRIO_MIN && iPrio <= BARCO_WARNING_PRIO_MAX )
    sFrameSource = BARCO_VO_WARNING;
  else if ( iPrio >= BARCO_FAULT_PRIO_MIN && iPrio <= BARCO_FAULT_PRIO_MAX )
    sFrameSource = BARCO_VO_FAULT;
  
  dpQuery("SELECT '_original.._value' FROM 'sys_workstation_*.anlagen' WHERE '_original.._value' LIKE  \"*"+sTunnel+"*\"", dda);
  
  if ( !mappingHasKey(gmShownAlerts, sAlarmDp) )//mapping initialisieren
    gmShownAlerts[sAlarmDp] = makeDynMapping();

  for ( int i = 2; i <= dynlen(dda); i++ )
  {
    string sWorkstationDp = dpSubStr(dda[i][1], DPSUB_DP);;
    string sWorkstation = sWorkstationDp;
    
    strreplace(sWorkstation, "sys_workstation_", "");
    
    dyn_int diX, diY;
    dyn_int diWidth, diHeight;
    dyn_string dsId;
    string sWorkspace;
    dyn_string dsSources;
    string sSelectedVo;
    
    if ( !dpExists("barco_workstation_"+sWorkstation) )
      continue;
  
    dpGet("barco_workstation_"+sWorkstation+".workspace.visualObjects.id", dsId,
          "barco_workstation_"+sWorkstation+".workspace.visualObjects.x", diX,
          "barco_workstation_"+sWorkstation+".workspace.visualObjects.y", diY,
          "barco_workstation_"+sWorkstation+".workspace.visualObjects.width", diWidth,
          "barco_workstation_"+sWorkstation+".workspace.visualObjects.height", diHeight,
          "barco_workstation_"+sWorkstation+".workspace.visualObjects.sourceName", dsSources,
          "barco_workstation_"+sWorkstation+".workspace.visualObjects.selected", sSelectedVo,
          "barco_workstation_"+sWorkstation+".workspace.name", sWorkspace
          );
    //sortieren von links oben nach rechts unten
    dyn_dyn_anytype ddf;
    ddf[1] = dsId;
    ddf[2] = diY;
    ddf[3] = diX;
    ddf[4] = diWidth;
    ddf[5] = diHeight;
    ddf[6] = dsSources;

    dynDynTurn(ddf);
    dynDynSort(ddf, makeDynInt(3));
    dynDynSort(ddf, makeDynInt(2));
    dynDynTurn(ddf);

    dsId = ddf[1];
    diY = ddf[2];
    diX = ddf[3];
    diWidth = ddf[4];
    diHeight = ddf[5];
    dsSources = ddf[6];
    
    if ( iActState == 1 /*|| iActState == 3 || iActState == 4*/ ) //Rahmen anzeigen bei alarm kam unquittiert
    {
      int iSelectedIdx = dynContains(dsId, sSelectedVo);
      
      mapping mAlert;

      mAlert["ALARM_SOURCE"] = sFrameSource;
      mAlert["WORKSTATION"]  = sWorkstation;
      
      //schauen ob bereits eine alarmaufschaltung von diesem tunnel bereits ist
      for ( int j = 1; j <= dynlen(dsId); j++ )
      {
        if ( iSelectedIdx == j )//selektiertes visual object ueberspringen
          continue;
        
        if ( (dsSources[j] == BARCO_VO_ALARM || dsSources[j] == BARCO_VO_WARNING || dsSources[j] == BARCO_VO_FAULT) && strpos(dsId[j], "_frame") != -1 )//alarmrahmen vorhanden
        {
          string sIdFrame = dsId[j];
          string sIdVisualObject = dsId[j];
          strreplace(sIdVisualObject, "_frame", "");
          
          int iVisualObjctIdx = dynContains(dsId, sIdVisualObject);
          
          string sCamDp = dpSubStr(barco_getCamDpOfSourceName(dsSources[iVisualObjctIdx]), DPSUB_DP);
          
          string sTempTunnel = sCamDp==""?"":strsplit(sCamDp, "_")[1];
          
          if ( sTempTunnel == sTunnel )//hier ist bereits ein alarm von diesem tunnel aufgeschalten daher hier rahmen aufschalten
          {        
            int x = diX[j]+BARCO_FRAME_WIDHT;
            int y = diY[j]+BARCO_FRAME_WIDHT;
            int width = diWidth[j]-BARCO_FRAME_WIDHT*2;
            int height = diHeight[j]-BARCO_FRAME_WIDHT*2;
            
            mAlert["VO_ID"] = dsId[iVisualObjctIdx];
            dynAppend(gmShownAlerts[sAlarmDp], mAlert);
    
            barco_executeCommand(sWorkstation, "voSetSource "+sWorkspace+" "+dsId[iVisualObjctIdx]+"_frame "+sFrameSource);
            barco_executeCommand(sWorkstation, "voSetPosition "+sWorkspace+" "+dsId[iVisualObjctIdx]+" "+x+" "+y, false);
            barco_executeCommand(sWorkstation, "voSetSize "+sWorkspace+" "+dsId[iVisualObjctIdx]+" "+width+" "+height, false);
            barco_executeCommand(sWorkstation, "voSetMode "+sWorkspace+" "+dsId[iVisualObjctIdx]+"_frame full", false);
            
            barco_executeCommand(sWorkstation, "voSetTitle "+sWorkspace+" "+dsId[iVisualObjctIdx]+" "+sAlarmSource, false);
            barco_executeCommand(sWorkstation, "voSetSource "+sWorkspace+" "+dsId[iVisualObjctIdx]+" "+sAlarmSource, false);
            barco_executeCommand(sWorkstation, "voSetLabel "+sWorkspace+" "+dsVisualObjects[iVisualObjctIdx]+" "+"label", false);
            
            barco_executeCommand(sWorkstation, "updateVisualObjects "+sWorkspace+" "+dsId[iVisualObjctIdx]+"_frame "+dsId[iVisualObjctIdx], false);
            
            barco_updateWorkstationOverview(sWorkstation);
            return;
          }
        }
      }
      
      //schauen ob bereits eine kamera von diesem tunnel aufgeschalten ist
      for ( int j = 1; j <= dynlen(dsId); j++ )
      {
        if ( iSelectedIdx == j )//selektiertes visual object ueberspringen
          continue;
        
        if ( strpos(dsId[j], "_frame") != -1 )//keine rahmen objekte behandeln
          continue;
        

        string sIdFrame = dsId[j]+"_frame";
        string sIdVisualObject = dsId[j];
                   
        string sCamDp = dpSubStr(barco_getCamDpOfSourceName(dsSources[j]), DPSUB_DP);
          
        string sTempTunnel = sCamDp==""?"":strsplit(sCamDp, "_")[1];
          
        if ( sTempTunnel == sTunnel )//hier ist bereits ein alarm von diesem tunnel aufgeschalten daher hier rahmen aufschalten
        {        
          int x = diX[j]+BARCO_FRAME_WIDHT;
          int y = diY[j]+BARCO_FRAME_WIDHT;
          int width = diWidth[j]-BARCO_FRAME_WIDHT*2;
          int height = diHeight[j]-BARCO_FRAME_WIDHT*2;
          
          mAlert["VO_ID"] = dsId[j];
          dynAppend(gmShownAlerts[sAlarmDp], mAlert);
    
          barco_executeCommand(sWorkstation, "voSetSource "+sWorkspace+" "+dsId[j]+"_frame "+sFrameSource);
          barco_executeCommand(sWorkstation, "voSetPosition "+sWorkspace+" "+dsId[j]+" "+x+" "+y, false);
          barco_executeCommand(sWorkstation, "voSetSize "+sWorkspace+" "+dsId[j]+" "+width+" "+height, false);
          barco_executeCommand(sWorkstation, "voSetMode "+sWorkspace+" "+dsId[j]+"_frame full", false);
            
          barco_executeCommand(sWorkstation, "voSetTitle "+sWorkspace+" "+dsId[j]+" "+sAlarmSource, false);
          barco_executeCommand(sWorkstation, "voSetSource "+sWorkspace+" "+dsId[j]+" "+sAlarmSource, false);
          barco_executeCommand(sWorkstation, "voSetLabel "+sWorkspace+" "+dsId[j]+" "+"label", false);
            
          barco_executeCommand(sWorkstation, "updateVisualObjects "+sWorkspace+" "+dsId[j]+"_frame "+dsId[j], false);
            
          barco_updateWorkstationOverview(sWorkstation);
          return;
        }
        
      }
      
      //ein leeres visual object suchen und hier aufschalten
      for ( int j = 1; j <= dynlen(dsId); j++ )
      {
        if ( iSelectedIdx == j )//selektiertes visual object ueberspringen
          continue;
        
        if ( strpos(dsId[j], "_frame") != -1 )//keine rahmen objekte behandeln
          continue;
        

        string sIdFrame = dsId[j]+"_frame";
        string sIdVisualObject = dsId[j];
        string sSource = dsSources[j];
          
        if ( sSource == BARCO_VO_OFF )//erster leerer bildschirm
        {        
          int x = diX[j]+BARCO_FRAME_WIDHT;
          int y = diY[j]+BARCO_FRAME_WIDHT;
          int width = diWidth[j]-BARCO_FRAME_WIDHT*2;
          int height = diHeight[j]-BARCO_FRAME_WIDHT*2;
          
          mAlert["VO_ID"] = dsId[j];
          dynAppend(gmShownAlerts[sAlarmDp], mAlert);
    
          barco_executeCommand(sWorkstation, "voSetSource "+sWorkspace+" "+dsId[j]+"_frame "+sFrameSource);
          barco_executeCommand(sWorkstation, "voSetPosition "+sWorkspace+" "+dsId[j]+" "+x+" "+y, false);
          barco_executeCommand(sWorkstation, "voSetSize "+sWorkspace+" "+dsId[j]+" "+width+" "+height, false);
          barco_executeCommand(sWorkstation, "voSetMode "+sWorkspace+" "+dsId[j]+"_frame full", false);
            
          barco_executeCommand(sWorkstation, "voSetTitle "+sWorkspace+" "+dsId[j]+" "+sAlarmSource, false);
          barco_executeCommand(sWorkstation, "voSetSource "+sWorkspace+" "+dsId[j]+" "+sAlarmSource, false);
          barco_executeCommand(sWorkstation, "voSetLabel "+sWorkspace+" "+dsId[j]+" "+"label", false);
            
          barco_executeCommand(sWorkstation, "updateVisualObjects "+sWorkspace+" "+dsId[j]+"_frame "+dsId[j], false);
            
          barco_updateWorkstationOverview(sWorkstation);
          return;
        }      
      }
      
      //ersten bildschirm aufschalten
      for ( int j = 1; j <= dynlen(dsId); j++ )
      {
        if ( iSelectedIdx == j )//selektiertes visual object ueberspringen
          continue;
        
        if ( strpos(dsId[j], "_frame") != -1 )//keine rahmen objekte behandeln
          continue;
        

        string sIdFrame = dsId[j]+"_frame";
        string sIdVisualObject = dsId[j];
        string sSource = dsSources[j];
                 
        int x = diX[j]+BARCO_FRAME_WIDHT;
        int y = diY[j]+BARCO_FRAME_WIDHT;
        int width = diWidth[j]-BARCO_FRAME_WIDHT*2;
        int height = diHeight[j]-BARCO_FRAME_WIDHT*2;
        
        mAlert["VO_ID"] = dsId[j];
        dynAppend(gmShownAlerts[sAlarmDp], mAlert);
    
        barco_executeCommand(sWorkstation, "voSetSource "+sWorkspace+" "+dsId[j]+"_frame "+sFrameSource);
        barco_executeCommand(sWorkstation, "voSetPosition "+sWorkspace+" "+dsId[j]+" "+x+" "+y, false);
        barco_executeCommand(sWorkstation, "voSetSize "+sWorkspace+" "+dsId[j]+" "+width+" "+height, false);
        barco_executeCommand(sWorkstation, "voSetMode "+sWorkspace+" "+dsId[j]+"_frame full", false);
            
        barco_executeCommand(sWorkstation, "voSetTitle "+sWorkspace+" "+dsId[j]+" "+sAlarmSource, false);
        barco_executeCommand(sWorkstation, "voSetSource "+sWorkspace+" "+dsId[j]+" "+sAlarmSource, false);
        barco_executeCommand(sWorkstation, "voSetLabel "+sWorkspace+" "+dsId[j]+" "+"label", false);
            
        barco_executeCommand(sWorkstation, "updateVisualObjects "+sWorkspace+" "+dsId[j]+"_frame "+dsId[j], false);
            
        barco_updateWorkstationOverview(sWorkstation);
        return;     
      }
    }
    
  }
  if ( iActState == 2 || iActState == 0 ) //rahmen ausblenden alarm quittiert
  { 

    for ( int i = dynlen(gmShownAlerts[sAlarmDp]); i > 0; i-- )
    {
      string sWorkstation = gmShownAlerts[sAlarmDp][i]["WORKSTATION"];
      dyn_int diX, diY;
      dyn_int diWidth, diHeight;
      dyn_string dsId;
      string sWorkspace;
      dyn_string dsSources;
      string sSelectedVo;
  
      dpGet("barco_workstation_"+sWorkstation+".workspace.visualObjects.id", dsId,
            "barco_workstation_"+sWorkstation+".workspace.visualObjects.x", diX,
            "barco_workstation_"+sWorkstation+".workspace.visualObjects.y", diY,
            "barco_workstation_"+sWorkstation+".workspace.visualObjects.width", diWidth,
            "barco_workstation_"+sWorkstation+".workspace.visualObjects.height", diHeight,
            "barco_workstation_"+sWorkstation+".workspace.visualObjects.sourceName", dsSources,
            "barco_workstation_"+sWorkstation+".workspace.visualObjects.selected", sSelectedVo,
            "barco_workstation_"+sWorkstation+".workspace.name", sWorkspace
            );
      
      int iPos = dynContains(dsId, gmShownAlerts[sAlarmDp][i]["VO_ID"]);
      int iPosFrame = dynContains(dsId, gmShownAlerts[sAlarmDp][i]["VO_ID"]+"_frame");

      if ( iPos > 0 && iPosFrame > 0 )//fenster noch vorhanden
      {
        if ( dsSources[iPosFrame] == gmShownAlerts[sAlarmDp][i]["ALARM_SOURCE"] )//alarm immer noch aufgeschalten
        {
          int x = diX[iPosFrame];
          int y = diY[iPosFrame];
          int width = diWidth[iPosFrame];
          int height = diHeight[iPosFrame];
          barco_executeCommand(sWorkstation, "voSetPosition "+sWorkspace+" "+dsId[iPos]+" "+x+" "+y, false);
          barco_executeCommand(sWorkstation, "voSetSize "+sWorkspace+" "+dsId[iPos]+" "+width+" "+height, false);
          barco_executeCommand(sWorkstation, "voSetSource "+sWorkspace+" "+dsId[iPosFrame]+" "+BARCO_VO_SELECTED, false);
          barco_executeCommand(sWorkstation, "voSetLabel "+sWorkspace+" "+dsId[j]+" "+"label", false);
      
          barco_executeCommand(sWorkstation, "updateVisualObjects "+sWorkspace+" "+dsId[iPos]+"_frame "+dsId[iPos], false);
          
          barco_updateWorkstationOverview(sWorkstation);
        }
      }
      
      dynRemove(gmShownAlerts[sAlarmDp], i);
    }
  }
}

 
/** 
  Layout wechseln und selektion wechseln
  @param sDp1 
  @param sSelected 
  @param sDp2 
  @param sSelectedLayout 
*/
void barco_video_matrix_frameControllerCB(string sDp1, string sSelected,
                                          string sDp2, string sSelectedLayout)
{   
  string sDp = dpSubStr(sDp1, DPSUB_DP);
  string sHostname = sDp;
  
  strreplace(sHostname, "barco_workstation_", "");
  
  if ( !mappingHasKey(BARCO_SELECTED_VO, sHostname) )
    BARCO_SELECTED_VO[sHostname] = "";
 
   
  if ( mappingHasKey(BARCO_SELECTED_LAYOUT, sHostname) && BARCO_SELECTED_LAYOUT[sHostname] != sSelectedLayout )//layoutwechsel
  {
    delay(0,500);
    
    bool bLayoutSwitchRunning;
    dpGet(sDp+".workspace.layoutSwitchRunning", bLayoutSwitchRunning);
    
    while( bLayoutSwitchRunning )//warten bis layoutumschaltung fertig ist
    {
      dpGet(sDp+".workspace.layoutSwitchRunning", bLayoutSwitchRunning);
      delay(0,200);
    }
    delay(1);
    dpGet(sDp1, sSelected);
  }
  
  BARCO_SELECTED_LAYOUT[sHostname] = sSelectedLayout;
  
  dyn_int diX, diY;
  dyn_int diWidth, diHeight;
  dyn_string dsId;
  string sWorkspace;
  dyn_string dsSources;
  
  dpGet(sDp+".workspace.visualObjects.id", dsId,
        sDp+".workspace.visualObjects.x", diX,
        sDp+".workspace.visualObjects.y", diY,
        sDp+".workspace.visualObjects.width", diWidth,
        sDp+".workspace.visualObjects.height", diHeight,
        sDp+".workspace.visualObjects.sourceName", dsSources,
        sDp+".workspace.name", sWorkspace);
  
  
  int iPos = dynContains(dsId, sSelected+"_frame");//hole frame object
  
  if ( iPos > 0 )
  { 
    int x = diX[iPos]+BARCO_FRAME_WIDHT;
    int y = diY[iPos]+BARCO_FRAME_WIDHT;
    int width = diWidth[iPos]-BARCO_FRAME_WIDHT*2;
    int height = diHeight[iPos]-BARCO_FRAME_WIDHT*2;
  
    //selektierten auswaehlen
    barco_executeCommand(sHostname, "voSetSource "+sWorkspace+" "+sSelected+"_frame "+BARCO_VO_SELECTED);
    barco_executeCommand(sHostname, "voSetPosition "+sWorkspace+" "+sSelected+" "+x+" "+y, false);
    barco_executeCommand(sHostname, "voSetSize "+sWorkspace+" "+sSelected+" "+width+" "+height, false);
    barco_executeCommand(sHostname, "voSetMode "+sWorkspace+" "+sSelected+"_frame full", false);
    barco_executeCommand(sHostname, "voSetLabel "+sWorkspace+" "+sSelected+" "+"label", false);

    //alten deselektieren
    iPos = dynContains(dsId, BARCO_SELECTED_VO[sHostname]+"_frame");//hole frame object
    
    string sAdd = "";
  
    if ( iPos > 0 && BARCO_SELECTED_VO[sHostname] != sSelected )//fenster auch vorhanden
    {
      x = diX[iPos];
      y = diY[iPos];
      width = diWidth[iPos];
      height = diHeight[iPos];
      barco_executeCommand(sHostname, "voSetPosition "+sWorkspace+" "+BARCO_SELECTED_VO[sHostname]+" "+x+" "+y, false);
      barco_executeCommand(sHostname, "voSetSize "+sWorkspace+" "+BARCO_SELECTED_VO[sHostname]+" "+width+" "+height, false);
      
      sAdd = " "+BARCO_SELECTED_VO[sHostname]+"_frame "+BARCO_SELECTED_VO[sHostname];
    }
  
    barco_executeCommand(sHostname, "updateVisualObjects "+sWorkspace+" "+sSelected+"_frame "+sSelected + sAdd
                                                          , false);
    
    barco_updateWorkstationOverview(sHostname);
  }
  
  BARCO_SELECTED_VO[sHostname] = sSelected;  
}
