/**
  Barco-Video Schnittstellen Funktionen
  
  @author: Michael Schmit (BECC GmbH)
  @date:   2018
*/

#uses "stdTraffic_intern"
#uses "ExcelCtrlExt"

global const string BARCO_END_CHAR = "\r\n";
global const string BARCO_END_PROMT = "\n\n";
global string       BARCO_OWS_IP;
global int          BARCO_OWS_PORT;
global string       BARCO_OWS_USER;
global string       BARCO_OWS_PASSWORD;
global const float  BARCO_WORKSPACE_ZOOM_FACTOR = 0.07;
global mapping      BARCO_SOURCE_OF_DPNAME;
global mapping      BARCO_DPNAME_OF_SOURCE;
global mapping      BARCO_CAM_NUMS;
global mapping      BARCO_VIDEO_MATRIX;
global const string BARCO_VO_OFF      = "Source-leer";
global const string BARCO_VO_SELECTED = "Source-Auswahl";
global const string BARCO_VO_ALARM    = "Source-Alarm";
global const string BARCO_VO_WARNING  = "Source-Warnung";
global const string BARCO_VO_FAULT    = "Source-Stoerung";
global const int    BARCO_FRAME_WIDHT = 10;
global int          BARCO_ALARM_PRIO_MAX   = 104;
global int          BARCO_ALARM_PRIO_MIN   = 91;
global int          BARCO_WARNING_PRIO_MAX = 74;
global int          BARCO_WARNING_PRIO_MIN = 61;
global int          BARCO_FAULT_PRIO_MAX   = 44;
global int          BARCO_FAULT_PRIO_MIN   = 31;

global string gsActUser;

/** 
  Initialisiert die noetigen variablen z.b. infos aus der config
*/
void barco_initVars()
{
  string sConfig = getPath(DATA_REL_PATH, "barco/barco.cfg");
  
  if ( !isfile(sConfig) )
  {
    stdTraffic_intern_logWarning(__FILE__, "Konfig-Datei existiert nicht: " + "barco/barco.cfg");
    return;
  }
  
  BARCO_OWS_IP = stdTraffic_intern_paCfgReadValue(sConfig, "ows", "ip");
  BARCO_OWS_PORT = stdTraffic_intern_paCfgReadValue(sConfig, "ows", "port");
  BARCO_OWS_USER = stdTraffic_intern_paCfgReadValue(sConfig, "ows", "user");
  BARCO_OWS_PASSWORD = stdTraffic_intern_paCfgReadValue(sConfig, "ows", "password");
  
  stdTraffic_intern_logInfo(__FILE__, "IP Adresse OWS-Server: "+BARCO_OWS_IP);
  stdTraffic_intern_logInfo(__FILE__, "Port des OWS-Server: "+BARCO_OWS_PORT);
  
  string sVideoSourcenList = getPath(DATA_REL_PATH, "barco/videoquellen.xlsx");
  
  if ( !isfile(sVideoSourcenList) )
  {
    stdTraffic_intern_logWarning(__FILE__, "Videoliste existiert nicht: " + "barco/videoquellen.xlsx");
    return;
  }
  
  dyn_dyn_anytype ddaContent, ddaFormulars;
  
  stdExcel_readFile(sVideoSourcenList, ddaContent, ddaFormulars);//videoliste einlesen
  
  int iIndexDp = 7;
  int iIndexSourceName = 2;
  int iIndexCamNum = 4;
  int iIndexNextCam = 8;
  int iIndexZuorndung = 1;
  int iIndexKurztext = 3;
  mapping m;
  
  BARCO_SOURCE_OF_DPNAME = m;
  
  string sLastCam;
  
  for ( int i = 2; i <= dynlen(ddaContent); i++ )
  {
    if ( dpExists(ddaContent[i][iIndexDp]) )//Kamera dp existiert
    {
      if ( !mappingHasKey(BARCO_SOURCE_OF_DPNAME, ddaContent[i][iIndexDp]) )
        BARCO_SOURCE_OF_DPNAME[ddaContent[i][iIndexDp]] = m;
      
      if ( !mappingHasKey(BARCO_SOURCE_OF_DPNAME, ddaContent[i][iIndexNextCam]) )
        BARCO_SOURCE_OF_DPNAME[ddaContent[i][iIndexNextCam]] = m;
      
      BARCO_DPNAME_OF_SOURCE[ddaContent[i][iIndexSourceName]] = ddaContent[i][iIndexDp];
      
      BARCO_SOURCE_OF_DPNAME[ddaContent[i][iIndexDp]]["SOURCE_NAME"]  = ddaContent[i][iIndexSourceName];
      BARCO_SOURCE_OF_DPNAME[ddaContent[i][iIndexDp]]["CAM_NUMBER"]   = ddaContent[i][iIndexCamNum];
      BARCO_SOURCE_OF_DPNAME[ddaContent[i][iIndexDp]]["NEXT_CAM"]     = ddaContent[i][iIndexNextCam];
      BARCO_SOURCE_OF_DPNAME[ddaContent[i][iIndexDp]]["ZUORDNUNG"]     = ddaContent[i][iIndexZuorndung];
      BARCO_SOURCE_OF_DPNAME[ddaContent[i][iIndexDp]]["KURZTEXT"]     = ddaContent[i][iIndexKurztext];
      BARCO_SOURCE_OF_DPNAME[ddaContent[i][iIndexNextCam]]["PREV_CAM"]= ddaContent[i][iIndexDp];
      
      BARCO_CAM_NUMS[ddaContent[i][iIndexCamNum]] = ddaContent[i][iIndexSourceName];
    }
  }
  
  string sVideoMatrix = getPath(DATA_REL_PATH, "barco/videoMatrix.xlsx");
  
  if ( !isfile(sVideoMatrix) )
  {
    stdTraffic_intern_logWarning(__FILE__, "Videomatrix existiert nicht: " + "barco/videoMatrix.xlsx");
    return;
  }
  
  stdExcel_readFile(sVideoMatrix, ddaContent, ddaFormulars);//videomatrix einlesen
  
  for ( int i = 2; i <= dynlen(ddaContent); i++ )
  {    
    string sDp = ddaContent[i][1];
    
    if ( dpExists(sDp) )
    {
      BARCO_VIDEO_MATRIX[sDp] = m;
    
      BARCO_VIDEO_MATRIX[sDp]["CAM_DP"] = ddaContent[i][2];
      BARCO_VIDEO_MATRIX[sDp]["TEXT"] = ddaContent[i][3];
    }
  }
}

/** 
  Liefert den source namen des kamera dps zurueck
  @param sDp ... kamera dp
  @return 
*/
string barco_getSourceNameOfCamDp(string sDp)
{
  if ( mappingHasKey(BARCO_SOURCE_OF_DPNAME, sDp) )
    return BARCO_SOURCE_OF_DPNAME[sDp]["SOURCE_NAME"];
  else
    return "";
}

/** 
  Liefert die naechste kamera zurueck
  @param sDp ... kamera dp
  @return 
*/
string barco_getNextCamOfCamDp(string sDp)
{
  if ( mappingHasKey(BARCO_SOURCE_OF_DPNAME, sDp) )
    return BARCO_SOURCE_OF_DPNAME[sDp]["NEXT_CAM"];
  else
    return "";
}

/** 
  Liefert die vorige kamera zurueck
  @param sDp ... kamera dp
  @return 
*/
string barco_getPreviousCamOfCamDp(string sDp)
{
  if ( mappingHasKey(BARCO_SOURCE_OF_DPNAME, sDp) )
    return BARCO_SOURCE_OF_DPNAME[sDp]["PREV_CAM"];
  else
    return "";
}



/** 
  Liefert den dp namen der source zurueck
  @param sSourceName ... source Namen
  @return 
*/
string barco_getCamDpOfSourceName(string sSourceName)
{
  if ( mappingHasKey(BARCO_DPNAME_OF_SOURCE, sSourceName) )
    return BARCO_DPNAME_OF_SOURCE[sSourceName];
  else
    return "";
}
    
/** 
  fuehrt einen befehl aus
  @param sWorkstation ... hostname der workstation 
  @param sCommand ... befehl
  @param bREST ... REST Command 
  @return result
*/
string barco_executeCommand(string sWorkstation, string sCommand, bool bREST = false)
{
  sWorkstation = strtolower(sWorkstation);
  DebugFTN("BARCO", sCommand);
  string sApDp = "barco_workstation_"+sWorkstation;
  
  dpSetWait(sApDp+".request.command", sCommand,
            sApDp+".request.user", getUserName(),
            sApDp+".request.manNum", myManNum(),
            sApDp+".request.manType", myManType(),
            sApDp+".request.REST_CMD", bREST);
  
  dyn_string dsCond;
  dyn_anytype daCond;
  
  dyn_string dsRetDp;
  dyn_anytype daRet;
    
  bool bTimeout;
  
  string sResult;
  
  dsCond = makeDynString(sApDp+".response.command:_original.._value",
                         sApDp+".response.user:_original.._value",
                         sApDp+".response.manNum:_original.._value",
                         sApDp+".response.manType:_original.._value");
  
  daCond = makeDynAnytype(sCommand,
                          getUserName(),
                          (uint)myManNum(),
                          (uint)myManType());
  
  dsRetDp = makeDynString(sApDp+".response.result:_original.._value");
  
  dpWaitForValue(dsCond, daCond, dsRetDp, daRet, 5, bTimeout);//Auf ergebnis warten 
 
  if ( bTimeout ) 
    return "";
  
  if ( dynlen(daRet) < 1 )
    return "";
  
  string sResult = daRet[1];
  
  if ( strlen(sResult) > 0 )
    if ( sResult[strlen(sResult)-1] == BARCO_END_CHAR )//wenn das letzte zeichen ein endchar ist dann wegschneiden
      sResult = strrtrim(sResult, BARCO_END_CHAR);
  
  if ( strpos(sResult, "ERROR") != -1 )//Fehler aufgetreten
  {
    string sTemp = sResult;
    
    strreplace(sTemp, "\n", "");
    
    stdTraffic_intern_logWarning(__FUNCTION__, "cmd: "+sCommand+" result: "+sTemp);
  }
  return sResult;
}

/** 
  liefert alle source zurueck
  @param sWorkstation ... hostname der workstation 
  @return 
*/
dyn_mapping barco_getSources(string sWorkstation)
{
  string sResult = barco_executeCommand(sWorkstation, "sources", true);

  dyn_mapping dmResult = jsonDecode(sResult);
  
  return dmResult;
}

/** 
  liefert alle layouts zurueck
  @param sWorkstation ... hostname der workstation
  @return 
*/
dyn_string barco_getLayouts(string sWorkstation)
{
  dyn_string dsRet;
   
  string sResult = barco_executeCommand(sWorkstation, "getLayouts", false);

  strreplace(sResult, "\n", "");
  strreplace(sResult, ", ",",");
  
  dsRet = stringToDynString(sResult, ",");
  return dsRet;
}

/** 
  liefert den workspacenamen des layouts zurueck
  @param sLayoutName ... layout name
  @return 
*/
string barco_getLayoutWorkspace(string sWorkstation, string sLayoutName)
{
  string sRet;
  
  string sResult = barco_executeCommand(sWorkstation, "getLayoutWorkspace "+sLayoutName, false);
  
  strreplace(sResult, "\n", "");
  
  sRet = sResult;
  
  return sRet;
}

/** 
  Funktion setzt einen datenpunt. es wird auf wertaenderung geprueft
  @param sDp 
  @param aValue 
*/
void barco_dpSetOldNewComp(string sDp, anytype aValue)
{
  anytype aActValue;
  
  dpGet(sDp, aActValue);
  
  if ( aActValue != aValue )//wert hat sich geandert
    dpSet(sDp, aValue);
}

/** 
  Funktion setzt einen datenpunt. es wird auf wertaenderung geprueft
  @param dsDp 
  @param daValue 
*/
void barco_dpSetOldNewCompDyn(dyn_string dsDp, dyn_anytype daValue)
{
  dyn_anytype daActValue;

  dpGet(dsDp, daActValue);

  if ( daActValue != daValue )//wert hat sich geandert
    dpSet(dsDp, daValue);
}

/** 
  liefert alle workspaces zurueck
  @param sWorkstation ... hostname der workstation 
  @return 
*/
dyn_string barco_getWorkspaces(string sWorkstation)
{
//   string sResult = barco_executeCommand(sWorkstation, "workspaces", true);
// 
//   dyn_mapping dmResult = jsonDecode(sResult);
//   return dmResult;
  
  dyn_string dsRet;
   
  string sResult = barco_executeCommand(sWorkstation, "getWorkspaces", false);

  strreplace(sResult, "\n", "");
  strreplace(sResult, ", ",",");
  
  dsRet = stringToDynString(sResult, ",");
  return dsRet;  
}

/** 
  liefert die workspace groesse zurueck
  @param sWorkstation ... hostname der workstation 
  @param sWorkspace ... workspacename
  @return di[1] ... breite
          di[2] ... hoehe
*/
dyn_int barco_getWorkspaceSize(string sWorkstation, string sWorkspace)
{
  string sResult = barco_executeCommand(sWorkstation, "getWorkspaceDetails "+sWorkspace, false);

  strreplace(sResult, "\n", "");
  
  dyn_string dsTemp = strsplit(sResult, " ");
  
  dyn_int diSize = strsplit(dsTemp[2], "x");

  return diSize;  
}

/** 
  liefert alle visual objects eines workspaces zurueck
  @param sWorkstation ... hostname der workstation 
  @return 
*/
dyn_mapping barco_getVisualObjects(string sWorkstation)
{
  string sWorkspace = barco_getWorkspaceNameOfWorkstation(sWorkstation);
  
  if (sWorkspace == "")
    return makeDynMapping();
  
//   string sResult = barco_executeCommand(sWorkstation, "sessions/"+sWorkspace+"/content", true);
  string sResult = barco_executeCommand(sWorkstation, "getJSONVisualObjects "+sWorkspace, false);//simple api statt rest

  dyn_mapping dmResult = jsonDecode(sResult);

  return dmResult;
}

/** 
  lifert den workspace namen einer workstation zurueck
  @param sWorkstation ... hostname der workstation 
  @return 
*/
string barco_getWorkspaceNameOfWorkstation(string sWorkstation)
{
  string sWorkspace;
  
  sWorkstation = strtolower(sWorkstation);

  dpGet("barco_workstation_"+sWorkstation+".workspace.name", sWorkspace);
  
  return sWorkspace;
}

/** 
  funktion laedt das angegebene layout
  @param sWorkstation  ... hostname der workstation 
  @param sLayoutName ... layout name
*/
void barco_loadLayout(string sWorkstation, string sLayoutName)
{
  sWorkstation = strtolower(sWorkstation);
  
  dpSet("barco_workstation_"+sWorkstation+".workspace.layoutSwitchRunning", true);
  
  string sWorkspace = barco_getWorkspaceNameOfWorkstation(sWorkstation);

  barco_executeCommand(sWorkstation, "loadLayout "+sLayoutName, false);
  
  dyn_mapping dmObjects = barco_getVisualObjects(sWorkstation);
  
  dyn_string dsUpdate;
  
  for ( int i = 1; i <= dynlen(dmObjects); i++ )
  {
    if ( strpos(dmObjects[i]["id"], "_frame") != -1 )
    {
      string sId = dmObjects[i]["id"];
      
      barco_executeCommand(strtolower(getHostname()), "voSetSource "+sWorkspace+" "+sId+" "+BARCO_VO_SELECTED);
      
      dynAppend(dsUpdate, sId);
      strreplace(sId, "_frame", "");
      dynAppend(dsUpdate, sId);
    }
  }
  
  if ( dynlen(dsUpdate) > 0 )
    barco_executeCommand(strtolower(getHostname()), "updateVisualObjects "+sWorkspace+" "+dynStringToString(dsUpdate, " "), false);
  
  delay(2);
  barco_updateWorkstationOverview(getHostname());
  
  
  dpSet("barco_workstation_"+sWorkstation+".workspace.selectedLayout", sLayoutName);
  delay(1);
  
  
  dpSet("barco_workstation_"+sWorkstation+".workspace.layoutSwitchRunning", false);
}

/** 
  lifert die workstation namen eines workspaces zurueck
  @param sWorkspace ... workspace 
  @return 
*/
string barco_getWorkstationNameOfWorkspace(string sWorkspace)
{
  string sRet;
  dyn_dyn_anytype dda;

  dpQuery("SELECT ':_original.._value' FROM 'barco_workstation_*.workspace.name' WHERE ':_original.._value' == \""+sWorkspace+"\"", dda);
  sRet = dda[2][1];
  sRet = dpSubStr(sRet, DPSUB_DP);
  strreplace(sRet, "barco_workstation_","");
  return sRet;
}

/** 
  funktion fuehrt ein update der ansicht durch
  @param sWorkstation ... hostname der workstation  
*/
synchronized void barco_updateWorkstationOverview(string sWorkstation)
{
  sWorkstation = strtolower(sWorkstation);
  
  dyn_mapping dmObjects = barco_getVisualObjects(sWorkstation);
  
  string sDp = "barco_workstation_"+sWorkstation;
  
  dyn_float dfX;
  dyn_float dfY;
  dyn_float dfHeight;
  dyn_float dfWidth;
  dyn_string dsId;
  dyn_string dsZone;  
  dyn_string dsWindowTitle;
  dyn_string dsWindowType; 
  dyn_string dsComponentType;
  dyn_string dsMediaGroup;
  dyn_string dsDescription;
  dyn_string dsName;
  dyn_string dsMode;
  dyn_string dsLock;
  
  for ( int i = 1; i <= dynlen(dmObjects); i++ )
  {
    dynAppend(dfX, dmObjects[i]["x"]);
    dynAppend(dfY, dmObjects[i]["y"]);
    dynAppend(dfHeight, dmObjects[i]["height"]);
    dynAppend(dfWidth, dmObjects[i]["width"]);
    dynAppend(dsId, dmObjects[i]["id"]);
    dynAppend(dsZone, dmObjects[i]["zone"]);
    
    dynAppend(dsWindowTitle, dmObjects[i]["title"]);
    dynAppend(dsWindowType, dmObjects[i]["frame"]);
    
    dynAppend(dsComponentType, dmObjects[i]["type"]);
    
    if ( mappingHasKey(dmObjects[i],"mediagroup") )
      dynAppend(dsMediaGroup, dmObjects[i]["mediagroup"]);
    else
      dynAppend(dsMediaGroup, "");
    
    if ( mappingHasKey(dmObjects[i],"description") )
      dynAppend(dsDescription, dmObjects[i]["description"]);
    else
      dynAppend(dsDescription, "");
    
    if ( mappingHasKey(dmObjects[i],"source") )
      dynAppend(dsName, dmObjects[i]["source"]);
    else
      dynAppend(dsName, "");
    
    if ( mappingHasKey(dmObjects[i],"mode") )
      dynAppend(dsMode, dmObjects[i]["mode"]);
    else
      dynAppend(dsMode, "");

    if ( mappingHasKey(dmObjects[i],"locked") )
      dynAppend(dsLock, dmObjects[i]["locked"]);
    else
      dynAppend(dsLock, "");

  }
    
  dyn_string dsDps;
  dyn_anytype daValues;
  
  dynAppend(dsDps, sDp+".workspace.visualObjects.x"); daValues[1] = dfX;
  dynAppend(dsDps, sDp+".workspace.visualObjects.y"); daValues[2] = dfY;
  dynAppend(dsDps, sDp+".workspace.visualObjects.height"); daValues[3] = dfHeight;
  dynAppend(dsDps, sDp+".workspace.visualObjects.width"); daValues[4] = dfWidth;
  dynAppend(dsDps, sDp+".workspace.visualObjects.id"); daValues[5] = dsId;
  dynAppend(dsDps, sDp+".workspace.visualObjects.zone"); daValues[6] = dsZone;
  
  dynAppend(dsDps, sDp+".workspace.visualObjects.windowTitle"); daValues[7] = dsWindowTitle;
  dynAppend(dsDps, sDp+".workspace.visualObjects.windowType"); daValues[8] = dsWindowType;
  
  dynAppend(dsDps, sDp+".workspace.visualObjects.componentType"); daValues[9] = dsComponentType;
  dynAppend(dsDps, sDp+".workspace.visualObjects.mediaGroup"); daValues[10] = dsMediaGroup;
  dynAppend(dsDps, sDp+".workspace.visualObjects.description"); daValues[11] = dsDescription;
  dynAppend(dsDps, sDp+".workspace.visualObjects.sourceName"); daValues[12] = dsName;
  
  dynAppend(dsDps, sDp+".workspace.visualObjects.mode"); daValues[13] = dsMode;
  dynAppend(dsDps, sDp+".workspace.visualObjects.locked"); daValues[14] = dsLock;
  
  barco_dpSetOldNewCompDyn(dsDps, daValues);
  
  //fuer jedes fenster alarm, warnung, stoerung und selektiert fenster anlegen
  string sWorkspace = barco_getWorkspaceNameOfWorkstation(sWorkstation);

  dyn_string dsShownIds;
  dyn_string dsFrameIds;
  
  bool bChanged = false;

  
  for ( int i = 1; i <= dynlen(dsId); i++)
  {
    string sId = dsId[i];
    string sFrameId = sId+"_frame";
    
    if ( strpos(sId, "_frame") != -1 )//alarmfenster hier nicht mehr beandeln
      continue;
    else
    {
      dynAppend(dsShownIds, sId);
      dynAppend(dsFrameIds, sFrameId);
    }
    
    if ( !dynContains(dsId, sFrameId) )//alarmfenster noch nicht vorhanden
    {    
      barco_executeCommand(sWorkstation, "cloneVisualObject "+sWorkspace+" "+sId+" "+sFrameId+" "+sWorkspace);
      barco_executeCommand(sWorkstation, "voSetSource "+sWorkspace+" "+sFrameId+" "+BARCO_VO_SELECTED);
      barco_executeCommand(sWorkstation, "voSetMode "+sWorkspace+" "+sFrameId+" full");
      barco_executeCommand(sWorkstation, "voSetLabel "+sWorkspace+" "+sFrameId+" "+"noLabel", false);
      barco_executeCommand(sWorkstation, "voSetFrame "+sWorkspace+" "+sFrameId+" "+"frameless", false);
//       barco_executeCommand(sWorkstation, "voSetLocked "+sWorkspace+" "+sFrameId+" "+"locked", false);
      
      bChanged = true;
    } 
  }
  
  if ( dynlen(dsFrameIds) > 0 && bChanged )
  {
    barco_executeCommand(sWorkstation, "updateVisualObjects "+sWorkspace+" "+dynStringToString(dsFrameIds, " ")+" "+dynStringToString(dsShownIds, " "));
    barco_updateWorkstationOverview(sWorkstation);
  }

}

/** 
  schaltet die gewuenschte source auf dem selektieren visual object auf
  @param sWorkstation ... hostname der workstation
  @param dsVisualObjects ... visual objects
  @param dsSourceIds ... sourceIds
  @param dsTitle ... title
  @param dsFrame ... frame/frameless
  @return 
*/
void barco_switchSource(string sWorkstation, dyn_string dsVisualObjects, dyn_string dsSourceIds, dyn_string dsTitle, dyn_string dsFrame)
{
  sWorkstation = strtolower(sWorkstation);
  
  string sWorkspace = barco_getWorkspaceNameOfWorkstation(sWorkstation);
  
  dyn_int diX, diY;
  dyn_int diWidth, diHeight;
  dyn_string dsId;
  string sWorkspace;
  dyn_string dsSources;
  string sSelectedVo;
  
  dpGet("barco_workstation_"+sWorkstation+".workspace.visualObjects.id", dsId,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.x", diX,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.y", diY,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.width", diWidth,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.height", diHeight,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.sourceName", dsSources,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.selected", sSelectedVo,
        "barco_workstation_"+sWorkstation+".workspace.name", sWorkspace
        );

  for ( int i = 1; i <= dynlen(dsVisualObjects); i++ )
  {
    int iPos = dynContains(dsId, dsVisualObjects[i]);
    int iPosFrame = dynContains(dsId, dsVisualObjects[i]+"_frame");
    
    if ( iPos > 0 && iPosFrame > 0 )
    {
      if ( dsSources[iPosFrame] == BARCO_VO_ALARM ||
           dsSources[iPosFrame] == BARCO_VO_WARNING || 
           dsSources[iPosFrame] == BARCO_VO_FAULT )//alarmfenster loeschen
      {
        int x = diX[iPosFrame];
        int y = diY[iPosFrame];
        int width = diWidth[iPosFrame];
        int height = diHeight[iPosFrame];
        
        barco_executeCommand(sWorkstation, "voSetPosition "+sWorkspace+" "+dsId[iPos]+" "+x+" "+y, false);
        barco_executeCommand(sWorkstation, "voSetSize "+sWorkspace+" "+dsId[iPos]+" "+width+" "+height, false);
        barco_executeCommand(sWorkstation, "voSetSource "+sWorkspace+" "+dsId[iPosFrame]+" "+BARCO_VO_SELECTED, false);
      }
    }
    
    barco_executeCommand(sWorkstation, "voSetSource "+sWorkspace+" "+dsVisualObjects[i]+" "+dsSourceIds[i], false);
    barco_executeCommand(sWorkstation, "voSetTitle "+sWorkspace+" "+dsVisualObjects[i]+" "+dsTitle[i], false);
    barco_executeCommand(sWorkstation, "voSetFrame "+sWorkspace+" "+dsVisualObjects[i]+" "+dsFrame[i], false);
    barco_executeCommand(sWorkstation, "voSetLabel "+sWorkspace+" "+dsVisualObjects[i]+" "+"label", false);
    barco_executeCommand(sWorkstation, "voSetFrame "+sWorkspace+" "+dsVisualObjects[i]+" "+"frameless", false);
//     barco_executeCommand(sWorkstation, "voSetLocked "+sWorkspace+" "+dsVisualObjects[i]+" "+"locked", false);
  }
  
  barco_executeCommand(sWorkstation, "updateVisualObjects "+sWorkspace+" "+dynStringToString(dsVisualObjects, " "), false);
}

/**
 hier wird die work funktion fuer den userwechsel gestartet 
*/
void barco_initUserChange()
{
  delay(1);
  
  dpConnect("barco_userChangeCB", myUiDpName()+".UserName");
}

/** 
  @param sDp 
  @param sUser 
*/
void barco_userChangeCB(string sDp, string sUser)
{
  if ( gsActUser == sUser )//keine aenderung
    return;
  
  gsActUser = sUser;
  
  dyn_string dsShowSources;
  
  dpGet("sys_user_"+getUserName()+".defaultVideo", dsShowSources);
  
  string sWorkstation = strtolower(getHostname());
      
  dyn_string dsIds;
  dyn_int diX;
  dyn_int diY;
      
  dpGet("barco_workstation_"+sWorkstation+".workspace.visualObjects.id", dsIds,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.x", diX,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.y", diY);
      
  //sortierung
  dyn_dyn_anytype dda;
  dda[1] = dsIds;
  dda[2] = diY;
  dda[3] = diX;

  dynDynTurn(dda);
  dynDynSort(dda, makeDynInt(3));
  dynDynSort(dda, makeDynInt(2));
  dynDynTurn(dda);

  dsIds = dda[1];
  diY = dda[2];
  diX = dda[3];
  
  for ( int i = dynlen(dsIds); i > 0; i-- )
  {
    if ( strpos(dsIds[i], "_frame") != -1 )//keine frames speichern
    {
      dynRemove(dsIds, i);
    }
  }
  
  dyn_string dsSwitchSources;
  dyn_string dsFrames;
  dyn_string dsTitles;
  
  for ( int i = 1; i <= dynlen(dsShowSources); i++ )
  {
    if ( i <= dynlen(dsIds) )
    {
      dynAppend(dsSwitchSources, dsShowSources[i]);
      dynAppend(dsFrames, "frameless");
      dynAppend(dsTitles, dsShowSources[i]==BARCO_VO_OFF?"Aus":dsShowSources[i]);
    }
  }
  
  barco_switchSource(sWorkstation, dsIds, dsSwitchSources, dsTitles, dsFrames);
  barco_updateWorkstationOverview(getHostname());
}

/** 
  select next cam
*/
void barco_selectNextCamera()
{
  string sWorkstation = strtolower(getHostname());
  
  string sSelected;
  
  dyn_string dsIds;
  dyn_string dsSourcen;
  
  dpGet("barco_workstation_"+sWorkstation+".workspace.visualObjects.selected", sSelected,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.id", dsIds,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.sourceName", dsSourcen);
  
  int iPos = dynContains(dsIds, sSelected);
  
  if ( iPos > 0 )
  {  
    string sCamDp = barco_getCamDpOfSourceName(dsSourcen[iPos]);
    
    string sNextCamDp = barco_getNextCamOfCamDp(sCamDp);
    string sNextSource = barco_getSourceNameOfCamDp(sNextCamDp);
    
    barco_switchSource(sWorkstation, makeDynString(sSelected), makeDynString(sNextSource), makeDynString("frameless"), makeDynString(sNextSource));
    barco_updateWorkstationOverview(sWorkstation);
  }
}

/** 
  select previous cam
*/
void barco_selectPreviousCamera()
{
  string sWorkstation = strtolower(getHostname());
  
  string sSelected;
  
  dyn_string dsIds;
  dyn_string dsSourcen;
  
  dpGet("barco_workstation_"+sWorkstation+".workspace.visualObjects.selected", sSelected,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.id", dsIds,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.sourceName", dsSourcen);
  
  int iPos = dynContains(dsIds, sSelected);
  
  if ( iPos > 0 )
  {  
    string sCamDp = barco_getCamDpOfSourceName(dsSourcen[iPos]);
    
    string sNextCamDp = barco_getPreviousCamOfCamDp(sCamDp);
    string sNextSource = barco_getSourceNameOfCamDp(sNextCamDp);
    
    barco_switchSource(sWorkstation, makeDynString(sSelected), makeDynString(sNextSource), makeDynString("frameless"), makeDynString(sNextSource));
    barco_updateWorkstationOverview(sWorkstation);
  }
}

/** 
  select next visual object
*/
void barco_selectNextVisualObject()
{
  string sWorkstation = strtolower(getHostname());
  
  string sSelected;
      
  dyn_string dsIds;
  dyn_int diX;
  dyn_int diY;
      
  dpGet("barco_workstation_"+sWorkstation+".workspace.visualObjects.selected", sSelected,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.id", dsIds,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.x", diX,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.y", diY);
  
  for ( int i = dynlen(dsIds); i > 0 ; i-- )
  {
    if ( strpos(dsIds[i], "_frame") != -1 )
      continue;
    else
    {
      dynRemove(dsIds, i);
      dynRemove(diX, i);
      dynRemove(diY, i);
    }
  }
      
  //sortierung
  dyn_dyn_anytype dda;
  dda[1] = dsIds;
  dda[2] = diY;
  dda[3] = diX;

  dynDynTurn(dda);
  dynDynSort(dda, makeDynInt(3));
  dynDynSort(dda, makeDynInt(2));
  dynDynTurn(dda);

  dsIds = dda[1];
  diY = dda[2];
  diX = dda[3];

  int iPos = dynContains(dsIds, sSelected+"_frame");
  
  if ( iPos > 0 )
  {
    if ( (iPos+1) <= dynlen(dsIds) )
    {
      string sTemp = dsIds[iPos+1];
      
      strreplace(sTemp, "_frame", "");
      
      dpSet("barco_workstation_"+sWorkstation+".workspace.visualObjects.selected", sTemp);
    }
  }
}

/** 
  select previous visual object
*/
void barco_selectPreviousVisualObject()
{
  string sWorkstation = strtolower(getHostname());
  
  string sSelected;
      
  dyn_string dsIds;
  dyn_int diX;
  dyn_int diY;
      
  dpGet("barco_workstation_"+sWorkstation+".workspace.visualObjects.selected", sSelected,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.id", dsIds,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.x", diX,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.y", diY);
  
  for ( int i = dynlen(dsIds); i > 0 ; i-- )
  {
    if ( strpos(dsIds[i], "_frame") != -1 )
      continue;
    else
    {
      dynRemove(dsIds, i);
      dynRemove(diX, i);
      dynRemove(diY, i);
    }
  }
      
  //sortierung
  dyn_dyn_anytype dda;
  dda[1] = dsIds;
  dda[2] = diY;
  dda[3] = diX;

  dynDynTurn(dda);
  dynDynSort(dda, makeDynInt(3));
  dynDynSort(dda, makeDynInt(2));
  dynDynTurn(dda);

  dsIds = dda[1];
  diY = dda[2];
  diX = dda[3];

  int iPos = dynContains(dsIds, sSelected+"_frame");
  
  if ( iPos > 0 )
  {
    if ( (iPos-1) > 0 )
    {
      string sTemp = dsIds[iPos-1];
      
      strreplace(sTemp, "_frame", "");
      
      dpSet("barco_workstation_"+sWorkstation+".workspace.visualObjects.selected", sTemp);
    }
  }
}

/** 
  selektiert das angegebene visual object
  @param iNum 
*/
void barco_selectVisualObject(int iNum)
{
  string sWorkstation = strtolower(getHostname());
      
  dyn_string dsIds;
  dyn_int diX;
  dyn_int diY;
      
  dpGet("barco_workstation_"+sWorkstation+".workspace.visualObjects.id", dsIds,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.x", diX,
        "barco_workstation_"+sWorkstation+".workspace.visualObjects.y", diY);
  
  for ( int i = dynlen(dsIds); i > 0 ; i-- )
  {
    if ( strpos(dsIds[i], "_frame") != -1 )
      continue;
    else
    {
      dynRemove(dsIds, i);
      dynRemove(diX, i);
      dynRemove(diY, i);
    }
  }
      
  //sortierung
  dyn_dyn_anytype dda;
  dda[1] = dsIds;
  dda[2] = diY;
  dda[3] = diX;

  dynDynTurn(dda);
  dynDynSort(dda, makeDynInt(3));
  dynDynSort(dda, makeDynInt(2));
  dynDynTurn(dda);

  dsIds = dda[1];
  diY = dda[2];
  diX = dda[3];
  
  string sTemp = dsIds[iNum];
      
  strreplace(sTemp, "_frame", "");
      
  dpSet("barco_workstation_"+sWorkstation+".workspace.visualObjects.selected", sTemp);
}
